@@ -2,7 +2,7 @@
  * refit/main.c
  * Main code for the boot menu
  *
- * Copyright (c) 2006-2009 Christoph Pfisterer
+ * Copyright (c) 2006-2010 Christoph Pfisterer
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -82,9 +82,9 @@ static VOID AboutRefit(VOID)
 {
     if (AboutMenu.EntryCount == 0) {
         AboutMenu.TitleImage = BuiltinIcon(BUILTIN_ICON_FUNC_ABOUT);
-        AddMenuInfoLine(&AboutMenu, L"rEFIt Version 0.13");
+        AddMenuInfoLine(&AboutMenu, L"rEFIt Version 0.14");
         AddMenuInfoLine(&AboutMenu, L"");
-        AddMenuInfoLine(&AboutMenu, L"Copyright (c) 2006-2009 Christoph Pfisterer");
+        AddMenuInfoLine(&AboutMenu, L"Copyright (c) 2006-2010 Christoph Pfisterer");
         AddMenuInfoLine(&AboutMenu, L"Portions Copyright (c) Intel Corporation and others");
         AddMenuInfoLine(&AboutMenu, L"");
         AddMenuInfoLine(&AboutMenu, L"Running on:");
@@ -285,6 +285,28 @@ static LOADER_ENTRY * AddLoaderEntry(IN CHAR16 *LoaderPath, IN CHAR16 *LoaderTit
     
     // loader-specific submenu entries
     if (LoaderKind == 1) {          // entries for Mac OS X
+#if defined(EFIX64)
+        SubEntry = AllocateZeroPool(sizeof(LOADER_ENTRY));
+        SubEntry->me.Title        = L"Boot Mac OS X with a 64-bit kernel";
+        SubEntry->me.Tag          = TAG_LOADER;
+        SubEntry->LoaderPath      = Entry->LoaderPath;
+        SubEntry->VolName         = Entry->VolName;
+        SubEntry->DevicePath      = Entry->DevicePath;
+        SubEntry->UseGraphicsMode = Entry->UseGraphicsMode;
+        SubEntry->LoadOptions     = L"arch=x86_64";
+        AddMenuEntry(SubScreen, (REFIT_MENU_ENTRY *)SubEntry);
+
+        SubEntry = AllocateZeroPool(sizeof(LOADER_ENTRY));
+        SubEntry->me.Title        = L"Boot Mac OS X with a 32-bit kernel";
+        SubEntry->me.Tag          = TAG_LOADER;
+        SubEntry->LoaderPath      = Entry->LoaderPath;
+        SubEntry->VolName         = Entry->VolName;
+        SubEntry->DevicePath      = Entry->DevicePath;
+        SubEntry->UseGraphicsMode = Entry->UseGraphicsMode;
+        SubEntry->LoadOptions     = L"arch=i386";
+        AddMenuEntry(SubScreen, (REFIT_MENU_ENTRY *)SubEntry);
+#endif
+        
         if (!(GlobalConfig.DisableFlags & DISABLE_FLAG_SINGLEUSER)) {
             SubEntry = AllocateZeroPool(sizeof(LOADER_ENTRY));
             SubEntry->me.Title        = L"Boot Mac OS X in verbose mode";
@@ -296,6 +318,28 @@ static LOADER_ENTRY * AddLoaderEntry(IN CHAR16 *LoaderPath, IN CHAR16 *LoaderTit
             SubEntry->LoadOptions     = L"-v";
             AddMenuEntry(SubScreen, (REFIT_MENU_ENTRY *)SubEntry);
             
+#if defined(EFIX64)
+            SubEntry = AllocateZeroPool(sizeof(LOADER_ENTRY));
+            SubEntry->me.Title        = L"Boot Mac OS X in verbose mode (64-bit)";
+            SubEntry->me.Tag          = TAG_LOADER;
+            SubEntry->LoaderPath      = Entry->LoaderPath;
+            SubEntry->VolName         = Entry->VolName;
+            SubEntry->DevicePath      = Entry->DevicePath;
+            SubEntry->UseGraphicsMode = FALSE;
+            SubEntry->LoadOptions     = L"-v arch=x86_64";
+            AddMenuEntry(SubScreen, (REFIT_MENU_ENTRY *)SubEntry);
+            
+            SubEntry = AllocateZeroPool(sizeof(LOADER_ENTRY));
+            SubEntry->me.Title        = L"Boot Mac OS X in verbose mode (32-bit)";
+            SubEntry->me.Tag          = TAG_LOADER;
+            SubEntry->LoaderPath      = Entry->LoaderPath;
+            SubEntry->VolName         = Entry->VolName;
+            SubEntry->DevicePath      = Entry->DevicePath;
+            SubEntry->UseGraphicsMode = FALSE;
+            SubEntry->LoadOptions     = L"-v arch=i386";
+            AddMenuEntry(SubScreen, (REFIT_MENU_ENTRY *)SubEntry);
+#endif
+            
             SubEntry = AllocateZeroPool(sizeof(LOADER_ENTRY));
             SubEntry->me.Title        = L"Boot Mac OS X in single user mode";
             SubEntry->me.Tag          = TAG_LOADER;
@@ -1061,7 +1105,7 @@ RefitMain (IN EFI_HANDLE           ImageHandle,
     FinishTextScreen(FALSE);
     
     while (MainLoopRunning) {
-        MenuExit = RunMainMenu(&MainMenu, &ChosenEntry);
+        MenuExit = RunMainMenu(&MainMenu, GlobalConfig.DefaultSelection, &ChosenEntry);
         
         // We don't allow exiting the main menu with the Escape key.
         if (MenuExit == MENU_EXIT_ESCAPE)
